--[[ 
Title: GuildAnnouncer
Description: Officers type events/notices into chat and Center Screen announcments appear for guildies
Author: pills
Date: 2014-12-28
  ]]

-- GLOBALS
_GA = {}
_GA.prefs_defaults = {
	enabled = true,
    showme = true,
    myevents = {},
    last_update = nil,
    ad1 = "",
    ad2 = "",
    ad3 = "",
    ad4 = "",
    ad5 = "",
}
_GA.prefs = nil
_GA.original_message_received_handler = nil
_GA.GuildMap = {}
-- /GLOBALS

_GA.GUILD_INFO_INDEX_TO_GUILD_CHAT_INDEX_OFFSET = 2

function _GA.FindInCurrentRoster(gid, fromName)
    local last_gid = GUILD_ROSTER_MANAGER.guildId
    GUILD_ROSTER_MANAGER:SetGuildId(gid)
    local roster = GUILD_ROSTER_MANAGER.masterList
    local checkName = nil
    local member_info = nil
    for i,v in pairs(roster) do
        if v.online then
            if fromName:find("@") then
                checkName = v.displayName
            else
                checkName = v.characterName
            end
            if checkName == fromName then GUILD_ROSTER_MANAGER:SetGuildId(last_gid) return v end
        end
    end
    GUILD_ROSTER_MANAGER:SetGuildId(last_gid)
    return member_info
end

function _GA.ChatMessageReceived(eventCode, messageType, fromName, text)
	if not _GA.prefs.enabled then return end
    
    if not _GA.prefs.showme then
	   if fromName == GetDisplayName() then return end
    end
	
    local gid = _GA.GuildMap[GetDynamicChatChannelName(messageType)]
	if not gid then return end

    if not text:find("^%[ANNOUNCE%]") then return end

	local guild_member_info = _GA.FindInCurrentRoster(gid, fromName)
	if not guild_member_info then return end
	
    local rank_index = guild_member_info.rankIndex
	
	if not DoesGuildRankHavePermission(gid, rank_index, GUILD_PERMISSION_OFFICER_CHAT_WRITE) then return end
	
    local announcement = text:gsub("^%[ANNOUNCE%]", "")
    CENTER_SCREEN_ANNOUNCE:AddMessage(EVENT_SKILL_RANK_UPDATE, CSA_EVENT_COMBINED_TEXT, SOUNDS.OBJECTIVE_DISCOVERED, GetGuildName(gid), announcement, '/esoui/art/login/credits_ourosboros.dds', '/esoui/art/screens_app/interactkeyframe_center.dds', nil)
    -- set after adding message to queue to let prehook reset other announcements to the original color
    local channel_index = messageType - _GA.GUILD_INFO_INDEX_TO_GUILD_CHAT_INDEX_OFFSET
    local r, g, b = GetChatCategoryColor(channel_index)
    CENTER_SCREEN_ANNOUNCE.m_largeText:SetColor(r, g, b, 1)
end

function _GA.GuildMapUpdate()
	_GA.GuildMap = {}
	for gid=1, GetNumGuilds() do
		local real_gid = GetGuildId(gid)
    	_GA.GuildMap[GetGuildName(real_gid)] = real_gid
    end
end

function _GA.ResetAnnounceColor()
    CENTER_SCREEN_ANNOUNCE.m_largeText:SetColor(1, 1, 1, 1)
end

function _GA.OnMouseUp(control, button, upInside)
    if not _GA.prefs.enabled then return end
    if button == 2 then
        ClearMenu()
        AddMenuItem("Announce", function() 
                                    local announce = "[ANNOUNCE] "
                                    announce = announce..ZO_ChatWindowTextEntryEditBox:GetText()
                                    ZO_ChatWindowTextEntryEditBox:SetText(announce)
                                    ZO_ChatWindowTextEntryEditBox:TakeFocus()
                                end
                    )
        ShowMenu()
    end
end
    
function _GA.OverloadChatHandler(messageType, fromName, text, isFromCustomerService)
    if _GA.prefs.show_announce_text then
        text = _GA.StripAnnounceText(text)
    end
   return _GA.original_message_received_handler(messageType, fromName, text, isFromCustomerService)
end

function _GA.StripAnnounceText(text)
    return text:gsub("^%[ANNOUNCE%]", "")
end

function _GA.AchievmentAwarded(eventCode, name, points, id, link)
    _GA.prefs.myevents[GetTimeStamp()] = {event_type="ACHIEVMENT", event_link="link"}
end

function _GA.MedalAwarded(eventCode, name, texture, condition)
    _GA.prefs.myevents[GetTimeStamp()] = {event_type="MEDAL", event_name=name, event_condition=condition}
end

function _GA.LevelUpdated(eventCode, unitTag, level)
    _GA.prefs.myevents[GetTimeStamp()] = {event_type="LEVEL", event_level=level}
end

function _GA.VeteranRankUpdated(eventCode, unitTag, rank)
    _GA.prefs.myevents[GetTimeStamp()] = {event_type="RANK", event_rank=rank}
end

-- do all this when the addon is loaded
function _GA.Init(eventCode, addOnName)
	if addOnName ~= "GuildAnnouncer" then return end

	-- Event Registration
	EVENT_MANAGER:RegisterForEvent("GA_ChatMessageReceived", EVENT_CHAT_MESSAGE_CHANNEL, _GA.ChatMessageReceived)
	EVENT_MANAGER:RegisterForEvent("GA_GuildMapUpdate", EVENT_GUILD_DATA_LOADED, _GA.GuildMapUpdate)
    EVENT_MANAGER:RegisterForEvent("GA_AchievmentAwarded",  EVENT_ACHIEVEMENT_AWARDED, _GA.AchievmentAwarded)
    EVENT_MANAGER:RegisterForEvent("GA_MedalAwarded",  EVENT_MEDAL_AWARDED, _GA.MedalAwarded)
    EVENT_MANAGER:RegisterForEvent("GA_LevelUpdated",  EVENT_LEVEL_UPDATE, _GA.LevelUpdated)
    EVENT_MANAGER:RegisterForEvent("GA_VeteranRankUpdated",  EVENT_VETERAN_RANK_UPDATE, _GA.VeteranRankUpdated)

    ZO_PreHook(CENTER_SCREEN_ANNOUNCE, "AddMessage", _GA.ResetAnnounceColor)

	_GA.prefs = ZO_SavedVars:New("SV_GA_PREFS", 1, nil, _GA.prefs_defaults)
	--_GA.LibGuildInfo = LibStub("LibGuildInfo-1.0")

	local LAM = LibStub("LibAddonMenu-2.0")
	local panelData = {
         type = "panel",
         name = "GuildAnnouncer",
    }
	LAM:RegisterAddonPanel("GuildAnnouncer_Panel", panelData)
    local optionsData = {
        [1] = {
            type = "checkbox",
            name = "Guild Announcer Enabled",
            tooltip = "Check to Enable Announcments, Uncheck to Disable",
            getFunc = function() return _GA.prefs.enabled end,
            setFunc = function(value) _GA.prefs.enabled = value end,
        },
        [2] = {
            type = "checkbox",
            name = "Show Announcements I Make",
            tooltip = "Check to Show Center Screen Announcements for Announcements Made by You",
            getFunc = function() return _GA.prefs.showme end,
            setFunc = function(value) _GA.prefs.showme = value end,
        },
        [3] = {
            type = "checkbox",
            name = "Remove [ANNOUNCE] Tag From Chat",
            tooltip = "Check to Remove [ANNOUNCE] Tag From Chat Text",
            getFunc = function() return _GA.prefs.show_announce_text end,
            setFunc = function(value) _GA.prefs.show_announce_text = value end,
        },
        [4] = {
            type="submenu",
            name="Guild Recruitment",
            controls = {
                [1] = {
                    type = "editbox",
                    name = "Advert #1 = /ga1",
                    tooltip = "Text to enter into chatbox when /ga1 command executed",
                    getFunc = function() return _GA.prefs.ad1 end,
                    setFunc = function(value) _GA.prefs.ad1 = value end,
                    isMultiline = true,
                    width = "full",
                    default = "",
                },
                [2] = {
                    type = "editbox",
                    name = "Advert #2 = /ga2",
                    tooltip = "Text to enter into chatbox when /ga2 command executed",
                    getFunc = function() return _GA.prefs.ad2 end,
                    setFunc = function(value) _GA.prefs.ad2 = value end,
                    isMultiline = true,
                    width = "full",
                    default = "",
                },
                [3] = {
                    type = "editbox",
                    name = "Advert #3 = /ga3",
                    tooltip = "Text to enter into chatbox when /ga3 command executed",
                    getFunc = function() return _GA.prefs.ad3 end,
                    setFunc = function(value) _GA.prefs.ad3 = value end,
                    isMultiline = true,
                    width = "full",
                    default = "",
                },
                [4] = {
                    type = "editbox",
                    name = "Advert #4 = /ga4",
                    tooltip = "Text to enter into chatbox when /ga4 command executed",
                    getFunc = function() return _GA.prefs.ad4 end,
                    setFunc = function(value) _GA.prefs.ad4 = value end,
                    isMultiline = true,
                    width = "full",
                    default = "",
                },
                [5] = {
                    type = "editbox",
                    name = "Advert #5 = /ga5",
                    tooltip = "Text to enter into chatbox when /ga5 command executed",
                    getFunc = function() return _GA.prefs.ad5 end,
                    setFunc = function(value) _GA.prefs.ad5 = value end,
                    isMultiline = true,
                    width = "full",
                    default = "",
                },
            }
        },
    }
    LAM:RegisterOptionControls("GuildAnnouncer_Panel", optionsData)

    SLASH_COMMANDS['/ga1'] = function() ZO_ChatWindowTextEntryEditBox:SetText(_GA.prefs.ad1) end
    SLASH_COMMANDS['/ga2'] = function() ZO_ChatWindowTextEntryEditBox:SetText(_GA.prefs.ad2) end
    SLASH_COMMANDS['/ga3'] = function() ZO_ChatWindowTextEntryEditBox:SetText(_GA.prefs.ad3) end
    SLASH_COMMANDS['/ga4'] = function() ZO_ChatWindowTextEntryEditBox:SetText(_GA.prefs.ad4) end
    SLASH_COMMANDS['/ga5'] = function() ZO_ChatWindowTextEntryEditBox:SetText(_GA.prefs.ad5) end

     ZO_ChatWindowTextEntryEditBox:SetHandler("OnMouseUp", _GA.OnMouseUp)
     -- do some function overloading on event handelers to handle stripping of text in chat
     local chatHandlers = ZO_ChatSystem_GetEventHandlers()
     _GA.original_message_received_handler = chatHandlers[EVENT_CHAT_MESSAGE_CHANNEL]
     ZO_ChatSystem_AddEventHandler(EVENT_CHAT_MESSAGE_CHANNEL, _GA.OverloadChatHandler)

    _GA.GuildMapUpdate()
end

EVENT_MANAGER:RegisterForEvent("GuildAnnouncer_Init", EVENT_ADD_ON_LOADED , _GA.Init)